#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Response::ServerGroups;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Lang;
use PureMessage::ServerGroup qw (:all);
use PureMessage::RPC::Client; # client lib
use HTML::Entities;

our %text;
my $PAGE_WIDTH = 600;

my $WIDGET = new PureMessage::Manager::Widget;

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::ServerGroups',
                                'stderr_to_stdout' =>1
                              );

$r->add_help(text => "Help", tab => "servgrp", page => "mgr_servgrp_service" );

my $HOST = $in{host};
my $SERVICE = lc($in{service});

$r->title(text('services_status_title', encode_entities($HOST,'&<>"\/\''), encode_entities($SERVICE,'&<>"\/\'')));

my ($host_id, $host) = get_host($HOST);
my $rpc;

if ($host and $rpc = PureMessage::RPC::Client->new({$host_id=>$host})) {

    $rpc->call("Services.status", $SERVICE);
    unless ($rpc->error) {
        $rpc->wait();
    }

    if($rpc->error) {
        $r->raise_error("An Error Occured: " . $rpc->error());
        $r->error_dlg();
        $r->send;
        exit;
    }
        
    my $data = $rpc->data($HOST);
    $data = [$data] unless ref($data) eq 'ARRAY';
    my @services = @$data;
        
    if(!@services) {
        $r->raise_error("An Error Occured: " . $rpc->error());
        $r->error_dlg();
        $r->send;
        exit;
    }
    
    $data = call_actions($data);
    @services = @$data if @$data;
    
    foreach my $record ( @services) {
        $record = {"status" => "Unknown"} 
            unless defined $record and ref($record) eq "HASH";
        
        my %raw_data = %$record;
                
        my %cooked_data;
        my @flds;
        $cooked_data{Name} = $raw_data{name}
            if $raw_data{name};
        $cooked_data{PID}  = $raw_data{pid} 
            if $raw_data{running};
        $cooked_data{VMSize}  = $raw_data{vsz} 
            if $raw_data{vsz};
        $cooked_data{Children}  = $raw_data{count} 
            if $raw_data{count};
        $cooked_data{Summary}  = $raw_data{summary} 
            if $raw_data{summary};
        $cooked_data{Ready}  = $raw_data{ready} ? "Yes" : "No" 
            if $raw_data{ready};
        $cooked_data{Port}  = $raw_data{port} 
            if $raw_data{port};
        $cooked_data{AutoStart}  = $raw_data{autostart} ? "Yes" : "No" 
            if $raw_data{autostart};
        
        my @buttons;
        if ( $cooked_data{'Status'} = $raw_data{status} ) {
            push(@buttons, {type=>'text', name=>'refresh', text=>'Refresh'}) 
                if !$raw_data{running};
        }
        else {
            if ($raw_data{running} eq "") {
                $cooked_data{'Status'} = 'Unknown';
            } elsif ($raw_data{running}) {
                $cooked_data{'Status'} = 'Running';
            } else {
                $cooked_data{'Status'} = 'Stopped';
            }
            push(@buttons, 
                {type=>'image',icon=>'start', name=>'start',text=>text('send_start')}) 
                if !$raw_data{running};
            push(@buttons, 
                {type=>'image',icon=>'stop',name=>'stop',text=>text('send_stop')}) 
                if $raw_data{running};
            push(@buttons, 
                {type=>'image',icon=>'restart',name=>'restart',text=>text('send_restart')}) 
                if $raw_data{running};
        }
        
        push(@buttons, 
             {type=>'image',icon=>'edit',href=>"$host->{proto}://$host->{host}:$host->{port}",
              text=>text('manage_remote'), target=>'_new'});

        map { push @flds, $_ if exists $cooked_data{$_} } 
                qw(Name Status Ready PID Port AutoStart Summary VMSize Children);
        my $tb = $r->ToolBar(buttons => [ \@buttons ],);
        my $form = $r->addForm(
                                data    => \%cooked_data,
                                fields  => \@flds,
                                columns => 1,
                                toolbar => $tb,
                                width   => $PAGE_WIDTH,
                                state   => { host => $HOST,
                                           service => encode_entities($SERVICE,'&<>"\/\''),},
                                action  => 'service.cgi',
                                name    => 'body_data_' . $raw_data{name},
                               );
    }
} 
else {
    $r->raise_error(text('service_unable_to_rpc',encode_entities($HOST,'&<>"\/\'')));
    $r->error_dlg();
}

sub call_actions {
    my $status = shift;
    my $data = [];
    foreach ( @$status ) {
        my $name = $_->{name};
        my $q = {};
        if ($WIDGET->read_input($REQUEST, 'body_data_' . $name)) {
            $q = $WIDGET->result_data('body_data_' . $name);
        }
        if (my @ops = grep{$q->{$_}} qw(start stop restart) ) {
            foreach my $op (@ops) {
                delete $rpc->{error};
                $rpc->call("Services.$op", $SERVICE, $name);
                unless ($rpc->error()) {
                    $rpc->wait();
                }
                if($rpc->error) {
                    $r->raise_error(text('rpc_error',$rpc->error()));
                } 
                else {
                    $r->log(text('rpc_sent', ucfirst($op),$HOST), "$HOST", 'Action', \%in, 'ServerGroups/');
                    $r->addPaneBox( width   => $PAGE_WIDTH,
                                    content => text('rpc_sent',ucfirst($op),$HOST),
                                    padding => '<BR> <BR> ',
                                    name    => 'body_information', );
                    push @$data, { name   => $name, 
                                   type   => $SERVICE,
                                   status => text('request_status', $op),
                                 };
                }
            }
        }
    }
    return $data;
}

$r->send();


